package cn.iocoder.yudao.module.shipment.service.boxOrderCheckInfo;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.common.exception.ServiceException;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.code.CodeUtils;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.order.dal.dataobject.order.OrderDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderLabel.OrderLabelDO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderLocation.OrderLocationDO;
import cn.iocoder.yudao.module.order.enums.ErrorCodeConstants;
import cn.iocoder.yudao.module.order.service.order.OrderLocationService;
import cn.iocoder.yudao.module.order.service.order.OrderQueryService;
import cn.iocoder.yudao.module.order.service.order.OrderService;
import cn.iocoder.yudao.module.order.service.orderLabel.OrderLabelService;
import cn.iocoder.yudao.module.order.vo.order.OrderQueryVO;
import cn.iocoder.yudao.module.order.vo.orderLabel.OrderLabelBackVO;
import cn.iocoder.yudao.module.order.vo.orderLabel.OrderLabelQueryVO;
import cn.iocoder.yudao.module.shipment.convert.boxOrderCheckInfo.BoxOrderCheckInfoConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxMergePkgDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxPkgOrderDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxPreloadGoodsDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.boxOrderCheckInfo.BoxOrderCheckInfoDO;
import cn.iocoder.yudao.module.shipment.dal.mysql.boxOrderCheckInfo.BoxOrderCheckInfoMapper;
import cn.iocoder.yudao.module.shipment.dto.OrderCheckInfoDto;
import cn.iocoder.yudao.module.shipment.enums.AirShipmentStatusEnum;
import cn.iocoder.yudao.module.shipment.enums.TyStatueEnum;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.service.boxLoadInfo.BoxLoadInfoService;
import cn.iocoder.yudao.module.shipment.service.boxMergePkg.BoxMergePkgService;
import cn.iocoder.yudao.module.shipment.service.boxPkgOrder.BoxPkgOrderService;
import cn.iocoder.yudao.module.shipment.service.boxPreloadGoods.BoxPreloadGoodsService;
import cn.iocoder.yudao.module.shipment.vo.box.BoxOrderSearchVO;
import cn.iocoder.yudao.module.shipment.vo.boxLoadInfo.BoxLoadInfoOrderVO;
import cn.iocoder.yudao.module.shipment.vo.boxOrderCheckInfo.BoxOrderCheckInfoCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxOrderCheckInfo.BoxOrderCheckInfoQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxOrderCheckInfo.BoxOrderCheckInfoRemoveReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxOrderCheckInfo.BoxOrderCheckInfoUpdateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxPkgOrder.BoxPkgOrderCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadGoods.BoxPreloadGoodsQueryVO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.*;

/**
 * 理货和合包标签 Service 实现类
 *
 * @author 江楠
 */
@Service
@Validated
public class BoxOrderCheckInfoServiceImpl extends AbstractService<BoxOrderCheckInfoMapper, BoxOrderCheckInfoDO> implements BoxOrderCheckInfoService {

    @Resource
    private BoxOrderCheckInfoMapper boxOrderCheckInfoMapper;

    @Resource
    private BoxService boxService;

    @Resource
    private BoxPreloadGoodsService boxPreloadGoodsService;

    @Resource
    private OrderQueryService orderQueryService;

    @Resource
    private OrderService orderService;

    @Resource
    private OrderLabelService orderLabelService;

    @Resource
    private BoxPkgOrderService boxPkgOrderService;

    @Resource
    private BoxLoadInfoService boxLoadInfoService;

    @Resource
    private OrderLocationService orderLocationService;

    @Resource
    private BoxMergePkgService boxMergePkgService;

    @Override
    public Long createBoxOrderCheckInfo(BoxOrderCheckInfoCreateReqVO createReqVO) {
        Long shipmentId = createReqVO.getShipmentId();

        boxService.checkApproval(shipmentId, null);

        checkShipmentStepStatus(shipmentId,createReqVO.getType());

        // 插入
        BoxOrderCheckInfoDO boxOrderCheckInfoDO = BoxOrderCheckInfoConvert.INSTANCE.convert(createReqVO);

        String orderNumCode = createReqVO.getOrderNumCode();
        Long orderId = 0L;
        int position = orderNumCode.lastIndexOf("-");
        String orderNumber = "";
        String boxNumber = "";
        if(position >= 0) {
            boxNumber = orderNumCode.substring(position + 1);
            orderNumber = orderNumCode.substring(0, position);
            OrderDO orderDO = orderService.getOne(new LambdaQueryWrapperX<OrderDO>()
                    .eq(OrderDO::getOrderNo, orderNumber)
            );
            if(orderDO == null) {
                throw exception(cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.ORDER_CASE_NUM_NOT_EXIST);
            }
            //判断这个订单是不是已经预装订单里面的父订单
            orderId = orderDO.getOrderId();
        } else {
            throw exception(cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.ORDER_CASE_NUM_NOT_EXIST);
//            //扫描的是空运的合包箱号
//            BoxMergePkgDO boxMergePkgDO = boxMergePkgService.selectOne(new LambdaQueryWrapper<BoxMergePkgDO>()
//                    .eq(BoxMergePkgDO::getShipmentId, shipmentId)
//                    .eq(BoxMergePkgDO::getPkgNum, orderNumCode)
//            );
//            if (boxMergePkgDO != null) {
//                List<BoxPkgOrderDO> boxPkgOrderDOS = boxPkgOrderService.selectList(new LambdaQueryWrapper<BoxPkgOrderDO>()
//                        .eq(BoxPkgOrderDO::getPkgId, boxMergePkgDO.getId())
//                );
//                if (CollectionUtil.isNotEmpty(boxPkgOrderDOS)) {
//                    List<Long> orderIds = boxPkgOrderDOS.stream().map(BoxPkgOrderDO::getOrderId).collect(Collectors.toList());
//                    List<OrderDO> orderDOS = orderService.selectList(OrderDO::getOrderId, orderIds);
//                    String orderNos = orderDOS.stream().map(OrderDO::getOrderNo).collect(Collectors.joining(","));
//                    BoxOrderCheckInfoCreateReqVO boxOrderCheckInfoCreateReqVO = new BoxOrderCheckInfoCreateReqVO();
//                    boxOrderCheckInfoCreateReqVO.setShipmentId(shipmentId);
//                    boxOrderCheckInfoCreateReqVO.setOrderNo(orderNos);
//                    boxOrderCheckInfoCreateReqVO.setType(createReqVO.getType());
//                    batchCreate(boxOrderCheckInfoCreateReqVO);
//                    return 0L;
//                }
//            } else {
//
//            }
        }
        Set<Long> orderIdList = new HashSet<>();
        List<OrderDO> orderDOList = new ArrayList<>();
        if (createReqVO.getType() == 0) {//理货
            List<BoxPreloadGoodsDO> goodsDOList = getPreloadGoods(shipmentId, null);
            if(CollectionUtil.isEmpty(goodsDOList)) {
                throw exception(ErrorCodeConstants.ORDER_NOT_EXISTS);
            }
            orderIdList = goodsDOList.stream()
                    .map(BoxPreloadGoodsDO::getOrderId)
                    .collect(Collectors.toSet());
            //这个出货单下的所有订单
            orderDOList = orderQueryService.getOrderList(orderIdList);

            if(!orderIdList.contains(orderId)) {

                for (OrderDO item : orderDOList) {
                    String parentNumber = item.getParentNumber();
                    if(parentNumber != null && parentNumber.equals(orderNumber)) {
                        //是这个订单的父订单，判断这个子订单是不是01开头
                        String itemOrderNumber = item.getOrderNo();
                        String firstOrderNumber = orderNumber + "-01";
                        if(itemOrderNumber.equals(firstOrderNumber)) {
                            //这个子订单相当于原始订单
                            //修改这个箱号为拆单的箱号
                            orderNumCode = itemOrderNumber + "-" + boxNumber;
                            boxOrderCheckInfoDO.setOrderNumCode(orderNumCode);
                        }
                    }
                }
            }
        } else if (createReqVO.getType() == 1) {//合包
            List<BoxPkgOrderDO> boxPkgOrderDOS = boxPkgOrderService.selectList(BoxPkgOrderDO::getPkgId, createReqVO.getPkgId());
            orderIdList = boxPkgOrderDOS.stream().map(BoxPkgOrderDO::getOrderId).collect(Collectors.toSet());
            if(orderIdList.isEmpty() || CollectionUtil.isEmpty(orderIdList)){
                throw exception(ORDER_CASE_NUM_NOT_EXIST);
            }
            orderDOList = orderQueryService.getOrderList(orderIdList);
            boolean isParentOrder = false;
            if(!orderIdList.contains(orderId)) {
                for (OrderDO item : orderDOList) {
                    String parentNumber = item.getParentNumber();
                    if(parentNumber != null && parentNumber.equals(orderNumber)) {
                        isParentOrder = true;
                        //是这个订单的父订单，判断这个子订单是不是01开头
                        String itemOrderNumber = item.getOrderNo();
                        String firstOrderNumber = orderNumber + "-01";
                        if(itemOrderNumber.equals(firstOrderNumber)) {
                            //这个子订单相当于原始订单
                            //修改这个箱号为拆单的箱号
                            orderNumCode = itemOrderNumber + "-" + boxNumber;
                            boxOrderCheckInfoDO.setOrderNumCode(orderNumCode);
                        }
                    }
                }
                if (!isParentOrder) {
                    Long selectCount = boxPreloadGoodsService.selectCount(new LambdaQueryWrapper<BoxPreloadGoodsDO>()
                            .eq(BoxPreloadGoodsDO::getShipmentId, createReqVO.getShipmentId())
                            .eq(BoxPreloadGoodsDO::getOrderId, orderId)
                    );
                    if (selectCount == null || selectCount.longValue() == 0L) {
                        throw exception(ORDER_NOT_IN_BOX);
                    }
                }
            }
            //查询该订单号-标签号是否和订单已经绑定  绑定则报错
            BoxOrderCheckInfoDO boxOrderCheckInfoDO1 = boxOrderCheckInfoMapper.selectOne(new LambdaQueryWrapperX<BoxOrderCheckInfoDO>()
                    .eq(BoxOrderCheckInfoDO::getPkgId, createReqVO.getPkgId())
                    .eq(BoxOrderCheckInfoDO::getOrderNumCode, orderNumCode)
                    .eq(BoxOrderCheckInfoDO::getType, 1)
            );
            if (boxOrderCheckInfoDO1 != null) {
                BoxMergePkgDO boxMergePkg = boxMergePkgService.getBoxMergePkg(boxOrderCheckInfoDO1.getPkgId());
                throw exception(BOX_ORDER_CHECK_ALREADY_EXISTS,createReqVO.getOrderNumCode(),boxMergePkg.getPkgNum());
            }
            Long pkgOrderCount = boxPkgOrderService.selectCount(new LambdaQueryWrapperX<BoxPkgOrderDO>()
                    .eq(BoxPkgOrderDO::getPkgId, createReqVO.getPkgId())
                    .eq(BoxPkgOrderDO::getOrderId, orderId)
            );
            //若合包未装箱，则自动装箱
            if (pkgOrderCount == null || pkgOrderCount <= 0L) {
                BoxPkgOrderCreateReqVO orderCreateReqVO = new BoxPkgOrderCreateReqVO();
                orderCreateReqVO.setPkgId(createReqVO.getPkgId());
                orderCreateReqVO.setOrderId(createReqVO.getOrderId());
                boxPkgOrderService.createBoxPkgOrder(orderCreateReqVO);
            }
        }else if(createReqVO.getType()==2){ //装箱
               System.out.print("");
        }
        //查找这个是否已经装了

        BoxOrderCheckInfoDO existDO = getOne(new LambdaQueryWrapper<BoxOrderCheckInfoDO>()
                .eq(BoxOrderCheckInfoDO::getOrderNumCode, orderNumCode)
                .eq(BoxOrderCheckInfoDO::getType,createReqVO.getType())
        );
        if(existDO != null) {
            if (createReqVO.getType() == 0) {
                throw exception(cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.ORDER_CASE_NUM_ALREADY_TALLY);
            } else {
                throw exception(cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_LOAD_EXISTS);
            }
        }

        Long resultOrderId = searchOrderIdByOrderNumCode(orderNumCode, orderIdList, orderDOList);

        if(resultOrderId == null) {
            throw exception(cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_LOAD_NOT_EXISTS);
        }

        boxOrderCheckInfoDO.setOrderId(resultOrderId);

        boxOrderCheckInfoMapper.insert(boxOrderCheckInfoDO);
        // 返回

        if(createReqVO.getType() == 0) {
            boxService.updateTallyStatus(shipmentId, Collections.singletonList(resultOrderId));
        }
        return boxOrderCheckInfoDO.getId();
    }

    @Override
    public void updateBoxOrderCheckInfo(BoxOrderCheckInfoUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateBoxOrderCheckInfoExists(updateReqVO.getId());
        // 更新
        BoxOrderCheckInfoDO updateObj = BoxOrderCheckInfoConvert.INSTANCE.convert(updateReqVO);
        boxOrderCheckInfoMapper.updateById(updateObj);
    }

    @Override
    public void deleteBoxOrderCheckInfo(Long id) {
        // 校验存在
        this.validateBoxOrderCheckInfoExists(id);
        // 删除
        boxOrderCheckInfoMapper.deleteById(id);
    }

    private void validateBoxOrderCheckInfoExists(Long id) {
        if (boxOrderCheckInfoMapper.selectById(id) == null) {
            throw exception(BOX_ORDER_CHECK_INFO_NOT_EXISTS);
        }
    }

    @Override
    public BoxOrderCheckInfoDO getBoxOrderCheckInfo(Long id) {
        return boxOrderCheckInfoMapper.selectById(id);
    }

    @Override
    public List<BoxOrderCheckInfoDO> getBoxOrderCheckInfoList(Collection<Long> ids) {
        return boxOrderCheckInfoMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<BoxOrderCheckInfoDO> getBoxOrderCheckInfoPage(BoxOrderCheckInfoQueryVO query, PageVO page) {
        return boxOrderCheckInfoMapper.selectPage(page, query);
    }

    @Override
    public List<BoxOrderCheckInfoDO> getBoxOrderCheckInfoList(BoxOrderCheckInfoQueryVO query) {
        return boxOrderCheckInfoMapper.selectList(query);
    }

    @Override
    public Boolean batchCreate(BoxOrderCheckInfoCreateReqVO createReqVO) {
        boolean flag = false;
        boxService.checkApproval(createReqVO.getShipmentId(), null);


        List<BoxOrderCheckInfoDO> boxOrderCheckInfoDOS = new ArrayList<>();

        String orderNos = createReqVO.getOrderNo();
        Long shipmentId = createReqVO.getShipmentId();

        checkShipmentStepStatus(shipmentId,createReqVO.getType());
        //支持批量多订单
        String[] orderNoArray = orderNos.split(",");

        Set<Long> orderIdList = new HashSet<>();
        if (createReqVO.getType() == 0) {
            //根据shipment,找到这个出货单下的所有订单商品
            List<BoxPreloadGoodsDO> goodsDOList = getPreloadGoods(shipmentId, null);
            if (CollectionUtil.isEmpty(goodsDOList)) {
                throw exception(ErrorCodeConstants.ORDER_NOT_EXISTS);
            }

            //出货单下涉及到的订单ID,判断当前批量装柜的订单ID在不在此次出货单里面
            orderIdList = goodsDOList.stream()
                    .map(BoxPreloadGoodsDO::getOrderId)
                    .collect(Collectors.toSet());
        } else if (createReqVO.getType() == 1) {
            List<BoxPkgOrderDO> boxPkgOrderDOS = boxPkgOrderService.selectList(new LambdaQueryWrapperX<BoxPkgOrderDO>()
                    .eq(BoxPkgOrderDO::getPkgId, createReqVO.getPkgId())
            );
            orderIdList = boxPkgOrderDOS.stream().map(BoxPkgOrderDO::getOrderId).collect(Collectors.toSet());
        }

        for (String orderNo : orderNoArray) {
            OrderDO orderDO = getOrderByNo(orderNo);
            Long orderId = orderDO.getOrderId();

            if (!orderIdList.contains(orderId)) {
                throw exception(cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_LOAD_NOT_EXISTS);
            }

            //删除以装柜的编号
            boxOrderCheckInfoMapper.delete(new LambdaQueryWrapper<BoxOrderCheckInfoDO>()
                    .eq(BoxOrderCheckInfoDO::getOrderId, orderId)
                    .eq(createReqVO.getPkgId() != null && createReqVO.getPkgId().longValue() > 0L,BoxOrderCheckInfoDO::getPkgId,createReqVO.getPkgId())
                    .eq(BoxOrderCheckInfoDO::getShipmentId, shipmentId)
                    .eq(BoxOrderCheckInfoDO::getType,createReqVO.getType())
            );

            //找到这个订单所有需要装柜的编号
            List<String> tagList = getOrderTagList(orderDO);
            for (int i = 0; i < tagList.size(); i++) {
                String tag = tagList.get(i);
                BoxOrderCheckInfoDO boxOrderCheckInfoDO = new BoxOrderCheckInfoDO();
                boxOrderCheckInfoDO.setOrderId(orderId);
                if (createReqVO.getType() == 1) {
                    boxOrderCheckInfoDO.setPkgId(createReqVO.getPkgId());
                }
                boxOrderCheckInfoDO.setOrderNumCode(tag);
                boxOrderCheckInfoDO.setShipmentId(shipmentId);
                boxOrderCheckInfoDO.setType(createReqVO.getType());
                boxOrderCheckInfoDOS.add(boxOrderCheckInfoDO);
            }
            saveBatch(boxOrderCheckInfoDOS);
        }
        flag = true;
        if(createReqVO.getType() == 0) {
            boxService.updateTallyStatus(shipmentId, orderIdList);
        }
        return flag;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void orderTally(Collection<Long> orderIdList, Long shipmentId) {
        boxOrderCheckInfoMapper.delete(new LambdaQueryWrapper<BoxOrderCheckInfoDO>()
                .in(BoxOrderCheckInfoDO::getOrderId, orderIdList)
                .eq(BoxOrderCheckInfoDO::getShipmentId, shipmentId)
                .eq(BoxOrderCheckInfoDO::getType, 0)
        );
        List<OrderDO> orderList = orderQueryService.getOrderList(orderIdList);
        List<BoxOrderCheckInfoDO> boxOrderCheckInfoDOList = new ArrayList<>();
        for (OrderDO orderDO : orderList) {
            List<String> tagList = getOrderTagList(orderDO);
            for (String tag : tagList) {
                BoxOrderCheckInfoDO boxOrderCheckInfoDO = new BoxOrderCheckInfoDO();
                boxOrderCheckInfoDO.setOrderId(orderDO.getOrderId());
                boxOrderCheckInfoDO.setOrderNumCode(tag);
                boxOrderCheckInfoDO.setShipmentId(shipmentId);
                boxOrderCheckInfoDO.setType(0);
                boxOrderCheckInfoDOList.add(boxOrderCheckInfoDO);
            }
        }
        saveBatch(boxOrderCheckInfoDOList);
    }

    @Override
    public void deleteBoxLoadInfo(BoxOrderCheckInfoCreateReqVO deleteReq) {
        boxService.checkApproval(deleteReq.getShipmentId(), null);

        String orderNos = deleteReq.getOrderNo();
        Long shipmentId = deleteReq.getShipmentId();

        checkShipmentStepStatus(shipmentId,deleteReq.getType());

        checkShipmentStepStatus(shipmentId,deleteReq.getType());
        if(StringUtils.isNotEmpty(orderNos)) {

            Set<Long> orderIdList = new HashSet<>();
            if (deleteReq.getType() == 0) {
                //根据shipment,找到这个出货单下的所有订单商品
                List<BoxPreloadGoodsDO> goodsDOList = getPreloadGoods(shipmentId, null);
                if (CollectionUtil.isEmpty(goodsDOList)) {
                    throw exception(ErrorCodeConstants.ORDER_NOT_EXISTS);
                }

                //出货单下涉及到的订单ID,判断当前批量装柜的订单ID在不在此次出货单里面
                orderIdList = goodsDOList.stream()
                        .map(BoxPreloadGoodsDO::getOrderId)
                        .collect(Collectors.toSet());
            } else if (deleteReq.getType() == 1) {
                List<BoxPkgOrderDO> boxPkgOrderDOS = boxPkgOrderService.selectList(new LambdaQueryWrapperX<BoxPkgOrderDO>()
                        .eq(BoxPkgOrderDO::getPkgId, deleteReq.getPkgId())
                );
                orderIdList = boxPkgOrderDOS.stream().map(BoxPkgOrderDO::getOrderId).collect(Collectors.toSet());
            }

            //批量删除
            String[] orderNoArray = orderNos.split(",");
            for (String orderNo : orderNoArray) {
                OrderDO orderDO = getOrderByNo(orderNo);
                Long orderId = orderDO.getOrderId();

                if(!orderIdList.contains(orderId)) {
                    throw exception(cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_LOAD_NOT_EXISTS);
                }

                boxOrderCheckInfoMapper.delete(new LambdaQueryWrapper<BoxOrderCheckInfoDO>()
                        .eq(BoxOrderCheckInfoDO::getOrderId, orderId)
                        .eq(deleteReq.getPkgId() != null && deleteReq.getPkgId().longValue() > 0L,BoxOrderCheckInfoDO::getPkgId,deleteReq.getPkgId())
                        .eq(BoxOrderCheckInfoDO::getShipmentId, deleteReq.getShipmentId())
                        .eq(BoxOrderCheckInfoDO::getType,deleteReq.getType())
                );
            }
            if(deleteReq.getType() == 0) {
                boxService.updateTallyStatus(shipmentId, orderIdList);
            }
        } else {
            BoxOrderCheckInfoDO existDO = getOne(new LambdaQueryWrapper<BoxOrderCheckInfoDO>()
                    .eq(BoxOrderCheckInfoDO::getOrderNumCode, deleteReq.getOrderNumCode())
                    .eq(BoxOrderCheckInfoDO::getType,deleteReq.getType())
            );
            if(existDO == null) {
                throw exception(cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.BOX_LOAD_NOT_EXISTS);
            }
            // 删除
            boxOrderCheckInfoMapper.deleteById(existDO);
            if(deleteReq.getType() == 0) {
                boxService.updateTallyStatus(shipmentId, Collections.singletonList(existDO.getOrderId()));
            }
        }
    }

    @Override
    public CommonResult<BoxLoadInfoOrderVO> orderTagList(Long orderId,Integer type) {
        //找到这个订单所有需要装柜的编号
        OrderDO orderDO = orderService.getById(orderId);
        List<String> tagList = getOrderTagList(orderDO);
        int tagSize = tagList.size();
        //找到这个订单所有已装柜的编号
        List<BoxOrderCheckInfoDO> existList = boxOrderCheckInfoMapper.selectList(new LambdaQueryWrapper<BoxOrderCheckInfoDO>()
                .eq(BoxOrderCheckInfoDO::getOrderId, orderId)
                .eq(BoxOrderCheckInfoDO::getType,type)
        );
        List<String> loadList = existList.stream()
                .map(BoxOrderCheckInfoDO::getOrderNumCode)
                .collect(Collectors.toList());
        tagList.removeAll(loadList);
        BoxLoadInfoOrderVO boxLoadInfoOrderVO = new BoxLoadInfoOrderVO();
        boxLoadInfoOrderVO.setLoadList(loadList);
        if(loadList.size() >= tagSize) {
            //拆单后，已装的标签号不正确，和orderLabel表标签不匹配，这里先按数量兼容
            boxLoadInfoOrderVO.setUnLoadList(new ArrayList<>());
        } else {
            boxLoadInfoOrderVO.setUnLoadList(tagList);
        }
        return CommonResult.success(boxLoadInfoOrderVO);
    }

    @Override
    public OrderCheckInfoDto searchLoadOrderByBoxNo(BoxOrderSearchVO boxOrderSearchVO) {
        //根据shipment和secId，找到这个出货单部分下的所有订单商品
        OrderCheckInfoDto orderCheckInfoDto = new OrderCheckInfoDto();
        Long shipmentId = boxOrderSearchVO.getShipmentId();
        String orderNumCode = boxOrderSearchVO.getOrderNumCode();
        String orderNo = boxOrderSearchVO.getOrderNo();
        Long orderId = null;
        if (StringUtils.isNotBlank(orderNumCode)) {
            orderId = boxLoadInfoService.getOrderIdByOrderNumCode(shipmentId, orderNumCode);
            if (orderId == null) {
                orderId = getOrderIdByOrderNo(orderNumCode);
            }
            if (orderId == null) {
                List<BoxMergePkgDO> boxMergePkgDOS = boxMergePkgService.selectList(BoxMergePkgDO::getPkgNum, orderNumCode);
                if (boxMergePkgDOS != null) {
                    orderCheckInfoDto.setMergeOrder(true);
                    orderCheckInfoDto.setMergeOrderNum(boxMergePkgDOS.get(0).getPkgNum());
                    return orderCheckInfoDto;
                } else {
                    throw new ServiceException(500, "订单/合包箱不在此出货单中");
                }
            }
        } else if (StringUtils.isNotBlank(orderNo)) {
            orderId = getOrderIdByOrderNo(orderNo);
        }
        OrderDO orderDO = orderService.selectOne(OrderDO::getOrderId, orderId);
        BeanUtil.copyProperties(orderDO,orderCheckInfoDto);
        List<OrderLocationDO> orderLocationListByOrderId = orderLocationService.getOrderLocationListByOrderId(orderId);
        orderCheckInfoDto.setOrderLocationDOList(orderLocationListByOrderId);
        List<OrderLabelBackVO> labelListByOrder = orderLabelService.getLabelListByOrder(orderId);
        Map<String, Integer> orderTagMap = getOrderTagMap(orderDO, labelListByOrder);
        orderCheckInfoDto.setStartNum(orderTagMap.get("start"));
        orderCheckInfoDto.setEndNum(orderTagMap.get("end"));
        return orderCheckInfoDto;
    }

    @Override
    public void batchRemove(BoxOrderCheckInfoRemoveReqVO removeReqVO) {
        checkShipmentStepStatus(removeReqVO.getShipmentId(),removeReqVO.getType());
        Integer startNo = removeReqVO.getStartNo();
        Integer endNo = removeReqVO.getEndNo();
        if (endNo - startNo < 0) {
            throw new ServiceException(500, "结束编号要大于等于起始编号");
        }
        String orderNo = removeReqVO.getOrderNo();
        List<String> orderNumCodeList = new ArrayList<>();

        OrderDO orderDO = orderService.selectOne(OrderDO::getOrderNo, orderNo);
        if (orderDO != null ) {
            if (startNo.equals(endNo)) {
                orderNumCodeList.add(CodeUtils.createOrderTagNumber(orderNo, startNo));
            } else {
                for (int i = startNo; i <= endNo; i++) {
                    orderNumCodeList.add(CodeUtils.createOrderTagNumber(orderNo, i));
                }
            }
            boxOrderCheckInfoMapper.delete(new LambdaQueryWrapper<BoxOrderCheckInfoDO>()
                    .eq(BoxOrderCheckInfoDO::getShipmentId,removeReqVO.getShipmentId())
                    .eq(BoxOrderCheckInfoDO::getType,removeReqVO.getType())
                    .in(BoxOrderCheckInfoDO::getOrderNumCode,orderNumCodeList)
            );
            if(removeReqVO.getType() == 0) {
                boxService.updateTallyStatus(removeReqVO.getShipmentId(), Collections.singletonList(orderDO.getOrderId()));
            }
        } else {
            BoxMergePkgDO boxMergePkgDO = boxMergePkgService.selectOne(BoxMergePkgDO::getPkgNum, orderNo);
            boxOrderCheckInfoMapper.delete(new LambdaQueryWrapper<BoxOrderCheckInfoDO>()
                    .eq(BoxOrderCheckInfoDO::getShipmentId,removeReqVO.getShipmentId())
                    .eq(BoxOrderCheckInfoDO::getType,removeReqVO.getType())
                    .eq(BoxOrderCheckInfoDO::getPkgId,boxMergePkgDO.getId())
            );
        }
    }

    private Map<String,Integer> getOrderTagMap(OrderDO orderDO, List<OrderLabelBackVO> orderLabelList) {
        //从OrderLabel里面查找订单的箱号
        Map<String,Integer> startEndLableMap = new HashMap<>();
        int start = 0;
        int end = 0;
        if(CollectionUtil.isEmpty(orderLabelList)) {
            //没有存箱号，从订单的总箱数里面取
            int totalNum = orderDO.getSumNum();//总箱数
            start = 1;
            end = totalNum;
        } else {
            orderLabelList.sort(Comparator.comparing(OrderLabelBackVO::getId));
            for (OrderLabelBackVO orderLabelDO : orderLabelList) {
                start = orderLabelDO.getStart();
                end = orderLabelDO.getEnd();
            }
        }
        startEndLableMap.put("start",start);
        startEndLableMap.put("end",end);
        return startEndLableMap;
    }

    private Long getOrderIdByOrderNo(String orderNo) {
        //箱号没查到，查订单号
        OrderDO orderDO = orderService.getOne(new LambdaQueryWrapperX<OrderDO>()
                .eq(OrderDO::getOrderNo, orderNo)
                .last("limit 1")
        );
        if (orderDO != null) {
            return orderDO.getOrderId();
        }
        return null;
    }

    /**
     * 根据shipment和secId，找到这个出货单部分下的所有订单商品
     * @param shipmentId shipmentId
     * @param secId  secId
     * @return
     */
    private List<BoxPreloadGoodsDO> getPreloadGoods(Long shipmentId, Long secId) {
        BoxPreloadGoodsQueryVO queryVO = new BoxPreloadGoodsQueryVO();
        queryVO.setShipmentId(shipmentId);
        queryVO.setIsRemove(0);
        if(secId != null) {
            queryVO.setSecId(secId);
        }
        return boxPreloadGoodsService.getBoxPreloadGoodsList(queryVO);
    }

    private Long searchOrderIdByOrderNumCode(String orderNumCode, Set<Long> orderIdList, List<OrderDO> orderDOList) {
        Long orderId = null;

        OrderLabelQueryVO queryVO = new OrderLabelQueryVO();
        queryVO.setOrderIdList(orderIdList);

        List<OrderLabelDO> labelDOList = orderLabelService.getLabelList(queryVO);
        Map<Long, List<OrderLabelDO>> labelMap = labelDOList.stream()
                .collect(Collectors.groupingBy(OrderLabelDO::getOrderId));

        for (OrderDO orderDO : orderDOList) {
            List<OrderLabelDO> orderLabelDOList = labelMap.get(orderDO.getOrderId());
            List<String> tagList = getOrderTagList(orderDO, orderLabelDOList);
            if(tagList.contains(orderNumCode)) {
                orderId = orderDO.getOrderId();
            }
        }
        return orderId;
    }

    private List<String> getOrderTagList(OrderDO orderDO, List<OrderLabelDO> orderLabelList) {
        //从OrderLabel里面查找订单的箱号
        String orderNo = orderDO.getOrderNo();
        List<String> tagList = new ArrayList<>();
        int start;
        int end;
        if(CollectionUtil.isEmpty(orderLabelList)) {
            //没有存箱号，从订单的总箱数里面取
            int totalNum = orderDO.getSumNum();//总箱数
            start = 1;
            end = totalNum;
            for (int i = start; i <= end; i++) {
                String tag = CodeUtils.createOrderTagNumber(orderNo, i);
                tagList.add(tag);
            }
        } else {
            orderLabelList.sort(Comparator.comparing(OrderLabelDO::getId));
            for (OrderLabelDO orderLabelDO : orderLabelList) {
                start = orderLabelDO.getStart();
                end = orderLabelDO.getEnd();
                for (int i = start; i <= end; i++) {
                    String tag = CodeUtils.createOrderTagNumber(orderNo, i);
                    tagList.add(tag);
                }
            }
        }

        return tagList;
    }

    private List<BoxOrderCheckInfoDO> getOrderTagNumCaseList(OrderDO orderDO, List<OrderLabelDO> orderLabelList) {
        //从OrderLabel里面查找订单的箱号
        String orderNo = orderDO.getOrderNo();
        List<BoxOrderCheckInfoDO> tagList = new ArrayList<>();
        int start;
        int end;
        if(CollectionUtil.isEmpty(orderLabelList)) {
            //没有存箱号，从订单的总箱数里面取
            int totalNum = orderDO.getSumNum();//总箱数
            start = 1;
            end = totalNum;
            for (int i = start; i <= end; i++) {
                String tag = CodeUtils.createOrderTagNumber(orderNo, i);
                BoxOrderCheckInfoDO checkInfoDO = new BoxOrderCheckInfoDO() ;
                checkInfoDO.setOrderId(orderDO.getOrderId());
                checkInfoDO.setOrderNumCode(tag) ;
                tagList.add(checkInfoDO);
            }
        } else {
            orderLabelList.sort(Comparator.comparing(OrderLabelDO::getId));
            for (OrderLabelDO orderLabelDO : orderLabelList) {
                start = orderLabelDO.getStart();
                end = orderLabelDO.getEnd();
                for (int i = start; i <= end; i++) {
                    String tag = CodeUtils.createOrderTagNumber(orderNo, i);
                    BoxOrderCheckInfoDO checkInfoDO = new BoxOrderCheckInfoDO() ;
                    checkInfoDO.setOrderId(orderDO.getOrderId());
                    checkInfoDO.setOrderNumCode(tag) ;
                    tagList.add(checkInfoDO);
                }
            }
        }

        return tagList;
    }

    private OrderDO getOrderByNo(String orderNo) {
        //根据订单编号找到订单
        OrderQueryVO orderQueryVO = new OrderQueryVO();
        orderQueryVO.setOrderNo(orderNo);
        List<OrderDO> orderList = orderQueryService.getOrderList(orderQueryVO);
        if(CollectionUtil.isEmpty(orderList)) {
            throw exception(ErrorCodeConstants.ORDER_NOT_EXISTS);
        }
        return orderList.get(0);
    }

    public List<String> getOrderTagList(OrderDO orderDO) {
        //从OrderLabel里面查找订单的箱号
        Long orderId = orderDO.getOrderId();
        OrderLabelQueryVO queryVO = new OrderLabelQueryVO();
        queryVO.setOrderId(orderId);
        List<OrderLabelDO> orderLabelList = orderLabelService.getLabelList(queryVO);
        return getOrderTagList(orderDO, orderLabelList);
    }


    @Override
    public List<BoxOrderCheckInfoDO> getOrderTagNumList(OrderDO orderDO) {
        //从OrderLabel里面查找订单的箱号
        Long orderId = orderDO.getOrderId();
        OrderLabelQueryVO queryVO = new OrderLabelQueryVO();
        queryVO.setOrderId(orderId);
        List<OrderLabelDO> orderLabelList = orderLabelService.getLabelList(queryVO);
        return getOrderTagNumCaseList(orderDO, orderLabelList);

    }

    private void checkShipmentStepStatus(Long shipmentId,Integer type) {
        BoxDO box = boxService.getBox(shipmentId);
        if (type == 0) {
            if (box.getTyStatus() == TyStatueEnum.HAS_TALLY.getTyStatus()) {
                throw exception(BOX_TALLY_FINISHED);
            }
        } else if (type == 1) {
            if (box.getAirShipmentStatus() != AirShipmentStatusEnum.UNSHIPMENT.getStatus()) {
                throw exception(BOX_MERGE_PKG_FINISHED);
            }
        }
    }
}
