package cn.iocoder.yudao.module.shipment.controller.admin;

import cn.iocoder.yudao.framework.common.exception.ServiceException;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.pdf.CustomXMLWorkerFontProvider;
import cn.iocoder.yudao.module.shipment.convert.BoxLadingBillConvert;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxLadingBillDO;
import cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants;
import cn.iocoder.yudao.module.shipment.service.boxLadingBill.BoxLadingBillService;
import cn.iocoder.yudao.module.shipment.vo.boxLadingBill.BoxLadingBillBackVO;
import cn.iocoder.yudao.module.shipment.vo.boxLadingBill.BoxLadingBillCreateReqVO;
import cn.iocoder.yudao.module.shipment.vo.boxLadingBill.BoxLadingBillQueryVO;
import cn.iocoder.yudao.module.shipment.vo.boxLadingBill.BoxLadingBillUpdateReqVO;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.itextpdf.text.pdf.*;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

@Validated
@RestController
@Api(tags = "管理后台 - 提单补料")
@RequestMapping("/ecw/box-lading-bill")
public class BoxLadingBillController {

    @Resource
    private BoxLadingBillService boxLadingBillService;

    @Value("${ue.templates-url}")
    private String path;

    @PostMapping("/create")
    @ApiOperation("创建提单补料, use this")
//    @PreAuthorize("@ss.hasPermission('ecw:box-lading-bill:create')")
    public CommonResult<Long> createBoxLadingBill(@Valid @RequestBody BoxLadingBillCreateReqVO createReqVO) {
        return success(boxLadingBillService.createBoxLadingBill(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新提单补料, use this")
//    @PreAuthorize("@ss.hasPermission('ecw:box-lading-bill:update')")
    public CommonResult<Boolean> updateBoxLadingBill(@Valid @RequestBody BoxLadingBillUpdateReqVO updateReqVO) {
        boxLadingBillService.updateBoxLadingBill(updateReqVO);
        return success(true);
    }


    @GetMapping("/get")
    @ApiOperation("获得提单补料")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-lading-bill:query')")
    public CommonResult<BoxLadingBillBackVO> getBoxLadingBill(@RequestParam("id") Long id) {
        BoxLadingBillDO boxLadingBill = boxLadingBillService.getBoxLadingBill(id);
        return success(BoxLadingBillConvert.INSTANCE.convert(boxLadingBill));
    }

    @GetMapping("/list")
    @ApiOperation("获得提单补料列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('ecw:box-lading-bill:query')")
    public CommonResult<List<BoxLadingBillBackVO>> getBoxLadingBillList(@RequestParam("ids") Collection<Long> ids) {
        List<BoxLadingBillDO> list = boxLadingBillService.getBoxLadingBillList(ids);
        return success(BoxLadingBillConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得提单补料分页")
    @PreAuthorize("@ss.hasPermission('ecw:box-lading-bill:query')")
    public CommonResult<PageResult<BoxLadingBillBackVO>> getBoxLadingBillPage(@Valid BoxLadingBillQueryVO query, PageVO page) {
        PageResult<BoxLadingBillDO> pageResult = boxLadingBillService.getBoxLadingBillPage(query, page);
        return success(BoxLadingBillConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出提单补料 Excel")
    @PreAuthorize("@ss.hasPermission('ecw:box-lading-bill:export')")
    @OperateLog(type = EXPORT)
    public void exportBoxLadingBillExcel(@Valid BoxLadingBillQueryVO query,
              HttpServletResponse response) throws IOException {
        List<BoxLadingBillDO> list = boxLadingBillService.getBoxLadingBillList(query);
        // 导出 Excel
        List<BoxLadingBillBackVO> datas = BoxLadingBillConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "提单补料.xls", "数据", BoxLadingBillBackVO.class, datas);
    }

    @GetMapping("/getBoxLadingBillPdf")
    @ApiOperation("获得提单补料pdf")
    @ApiImplicitParam(name = "shipmentId", value = "出货编号", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<String> getBoxLadingBillPdf(@RequestParam("shipmentId") Long shipmentId,@RequestParam("type") Integer type,HttpServletResponse response) throws Exception {
        InputStream inputStream = null;
        OutputStream outputStream = null;
        BoxLadingBillDO boxLadingBillDO = boxLadingBillService.selectOne(BoxLadingBillDO::getShipmentId, shipmentId);
        if (boxLadingBillDO == null) {
            return error(ErrorCodeConstants.LADING_BILL_NOT_EXIST);
        }
        try {
//            inputStream = new FileInputStream(new File("C:\\lzlj\\boxLadingBill.xlsx"));
            if(type==1){//从海运页面进入
                inputStream = this.getClass().getClassLoader().getResourceAsStream(path + "/boxLadingBill.xlsx");
            }else if(type==2){
                inputStream = this.getClass().getClassLoader().getResourceAsStream(path + "/boxAirLadingBill.xlsx");
            }

            outputStream = response.getOutputStream();
            Map<String, String> boxLadingBillPdf = boxLadingBillService.getBoxLadingBillPdf(shipmentId);
            ExcelWriter excelWriter = EasyExcel.write(outputStream).withTemplate(inputStream).build();
            WriteSheet sheet = EasyExcel.writerSheet(0).build();
            sheet.setSheetName("空运");
            excelWriter.fill(boxLadingBillPdf, sheet);
            excelWriter.finish();
            outputStream.flush();
        }  finally {
            if (inputStream != null) {
                inputStream.close();
            }
            if (outputStream != null) {
                outputStream.close();
            }
        }
        return success("");
    }
}
